import {
  ConflictException,
  ForbiddenException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Carrito } from './entity/carrito.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { Operador } from 'src/operador/entity/operador.entity';
import { TipoCarrito } from '../institucion-tipo-carrito/entity/tipo-carrito.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { MarcaService } from '../marca/marca.service';
import { ModeloService } from '../modelo/modelo.service';
import { ModuloService } from '../modulo/modulo.service';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    private institucionService: InstitucionService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private marcaService: MarcaService,
    private modeloService: ModeloService,
    private moduloService: ModuloService,
  ) {}

  async create(
    admin: Operador,
    id_modulo: number,
    id_tipo_carrito: number,
    id_marca: number,
    id_modelo: number,
  ) {
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findTipoCarritoById(
        id_tipo_carrito,
      );
    const marca = await this.marcaService.findById(id_marca, 'c');
    const modelo = await this.modeloService.findById(id_modelo, 'c');

    if (admin.institucion.id_institucion != modulo.institucion.id_institucion)
      throw new ForbiddenException(
        'No puedes crear un carrito para este módulo porque no pertenece a tu institución.',
      );
    return this.repository
      .count({ modulo, tipoCarrito })
      .then((carritos) =>
        this.repository.save(
          this.repository.create({
            carrito: `C${carritos >= 9 ? '' : '0'}${carritos + 1}`,
            marca,
            modelo,
            modulo,
            tipoCarrito,
          }),
        ),
      )
      .then((_) => ({
        message: 'Se creó correctamente un nuevo carrito para este módulo.',
      }));
  }

  async findAll(
    operador: Operador,
    filtros: {
      pagina: string;
      activo?: string;
      carrito?: string;
      id_institucion?: string;
      id_marca?: string;
      id_modelo?: string;
      id_modulo?: string;
      id_tipo_carrito?: string;
    },
  ) {
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    const marca = filtros.id_marca
      ? await this.marcaService.findById(parseInt(filtros.id_marca), 'c')
      : null;
    const modelo = filtros.id_modelo
      ? await this.modeloService.findById(parseInt(filtros.id_modelo), 'c')
      : null;
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCarritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;
    const query = this.repository
      .createQueryBuilder('c')
      .innerJoinAndSelect('c.marca', 'ma')
      .innerJoinAndSelect('c.modelo', 'mo')
      .innerJoinAndSelect('c.modulo', 'm')
      .innerJoinAndSelect('c.tipoCarrito', 'tc')
      .innerJoinAndSelect('m.institucion', 'i')
      .orderBy('i.institucion')
      .addOrderBy('m.modulo')
      .addOrderBy('tc.tipo_carrito')
      .addOrderBy('c.carrito')
      .skip((parseInt(filtros.pagina) - 1) * 25)
      .take(25);

    if (modulo) {
      if (
        operador.tipoUsuario.id_tipo_usuario > 2 &&
        operador.institucion.id_institucion != modulo.institucion.id_institucion
      )
        throw new ForbiddenException(
          'No puedes acceder a esta información porque este módulo no pertenece a tu institución.',
        );
      query.andWhere('m.id_modulo = :id_modulo', {
        id_modulo: modulo.id_modulo,
      });
    }
    if (filtros.activo === 'true') query.andWhere('c.activo = 1');
    if (filtros.activo === 'false') query.andWhere('c.activo = 0');
    if (filtros.carrito)
      query.andWhere('c.carrito LIKE :carrito', {
        carrito: `%${filtros.carrito}%`,
      });
    if (institucion && operador.tipoUsuario.id_tipo_usuario === 2)
      query.andWhere('i.id_institucion = :id_institucion', {
        id_institucion: institucion.id_institucion,
      });
    if (marca)
      query.andWhere('ma.id_marca = :id_marca', {
        id_marca: marca.id_marca,
      });
    if (modelo)
      query.andWhere('mo.id_modelo = :id_modelo', {
        id_modelo: modelo.id_modelo,
      });
    if (tipoCarrito)
      query.andWhere('tc.id_tipo_carrito = :id_tipo_carrito', {
        id_tipo_carrito: tipoCarrito.id_tipo_carrito,
      });
    return query.getManyAndCount();
  }

  findById(id_carrito: number) {
    return this.repository.findOne({ id_carrito }).then((carrito) => {
      if (!carrito) throw new NotFoundException('No existe este id carrito.');
      return carrito;
    });
  }

  async findCarrito(
    id_modulo: number | Modulo,
    id_tipo_carrito: number | TipoCarrito,
    carrito: string,
    validarNoExiste = true,
  ) {
    const modulo =
      typeof id_modulo === 'number'
        ? await this.moduloService.findById(id_modulo)
        : id_modulo;
    const tipoCarrito =
      typeof id_tipo_carrito === 'number'
        ? await this.institucionTipoCarritoService.findTipoCarritoById(
            id_tipo_carrito,
          )
        : id_tipo_carrito;

    return this.repository
      .findOne({ carrito, modulo, tipoCarrito })
      .then((carrito) => {
        if (validarNoExiste && !carrito)
          throw new ConflictException('No existe este carrito.');
        return carrito;
      });
  }

  async update(operador: Operador, attrs: Partial<Carrito>) {
    return this.findById(attrs.id_carrito)
      .then(async (carrito) => {
        if (
          operador.institucion.id_institucion !=
          carrito.modulo.institucion.id_institucion
        )
          throw new ForbiddenException(
            'No puedes modificar la información este carrito porque no pertenece a tu institución.',
          );
        Object.assign(carrito, attrs);
        return this.repository.save(carrito);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
