import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionTipoCarritoService } from './institucion-tipo-carrito.service';
import { Operador } from 'src/operador/entity/operador.entity';
import { Usuario } from 'src/usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { CreateInstitucionTipoCarritoDto } from './dto/input/create.dto';
import { UpdateInstitucionTipoCarritoDto } from './dto/input/update.dto';
import { InstitucionTiposCarritoOutputDto } from './dto/output/institucion-tipos-carrito.dto';
import { InstitucionTiposCarritoMostarOutputDto } from './dto/output/institucion-tipos-carrito-mostar.dto';
import { TipoCarritoOutputDto } from './dto/output/tipo-carrito.dto';

@Controller('institucion-tipo-carrito')
@ApiTags('institucion-tipo-carrito')
export class InstitucionTipoCarritoController {
  constructor(
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
  ) {}

  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que crea un tipo de carrito.' })
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { letra: '', tipo_carrito: '' } } },
  })
  @ApiBearerAuth('jwt')
  create(@Request() req, @Body() body: CreateInstitucionTipoCarritoDto) {
    const superAdmin: Operador = req.user.operador;

    if (!superAdmin || superAdmin.tipoUsuario.id_tipo_usuario != 2)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionTipoCarritoService.create(
      body.letra,
      body.tipo_carrito,
    );
  }

  @Serealize(TipoCarritoOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todos los tipos de carrito.',
  })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const operador: Operador = req.user.operador;

    if (
      !operador ||
      (operador.tipoUsuario.id_tipo_usuario != 2 &&
        operador.tipoUsuario.id_tipo_usuario != 3 &&
        operador.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionTipoCarritoService.findAll();
  }

  @Serealize(InstitucionTiposCarritoOutputDto)
  @Get('tipos-carrito')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna todos los tipos de carrito de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  tiposCarrito(@Request() req, @Query() query: IdInstitucionDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.institucionTipoCarritoService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Serealize(InstitucionTiposCarritoMostarOutputDto)
  @Get('tipos-carrito-mostrar')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna todos los tipos de carrito de una institución que tengan el campo mostrar en true.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  tiposCarritoMostrar(@Request() req, @Query() query: IdInstitucionDto) {
    const usuario: Usuario = req.user.usuario;

    if (!usuario)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.institucionTipoCarritoService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
      true,
    );
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que actualiza el campo mostrar de una institución tipo carrito.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: {
      ejemplo: { value: { id_institucion_tipo_carrito: 130, mostrar: true } },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionTipoCarritoDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionTipoCarritoService.update(admin, body);
  }
}
