import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionTipoEntradaService } from './institucion-tipo-entrada.service';
import { Operador } from 'src/operador/entity/operador.entity';
import { Usuario } from 'src/usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { CreateInstitucionTipoEntradaDto } from './dto/input/create.dto';
import { UpdateInstitucionTipoEntradaDto } from './dto/input/update.dto';
import { InstitucionTipoEntradaOutputDto } from './dto/output/institucion-tipo-entrada.dto';
import { InstitucionTipoEntradaMinOutputDto } from './dto/output/institucion-tipo-entrada-min.dto';
import { TipoEntradaOutputDto } from './dto/output/tipo-entrada.dto';

@Controller('institucion-tipo-entrada')
@ApiTags('institucion-tipo-entrada')
export class InstitucionTipoEntradaController {
  constructor(private tipoEntradaService: InstitucionTipoEntradaService) {}

  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que crea un tipo de entrada.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Es obligatorio mandar la variable tipo_entrada.',
    examples: { ejemplo: { value: { tipo_entrada: '' } } },
  })
  create(@Request() req, @Body() body: CreateInstitucionTipoEntradaDto) {
    const superAdmin: Operador = req.user.operador;

    if (!superAdmin || superAdmin.tipoUsuario.id_tipo_usuario != 2)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.tipoEntradaService.create(body.tipo_entrada);
  }

  @Serealize(TipoEntradaOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todos los tipos de entrada.',
  })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const operador: Operador = req.user.operador;

    if (
      !operador ||
      (operador.tipoUsuario.id_tipo_usuario != 2 &&
        operador.tipoUsuario.id_tipo_usuario != 3 &&
        operador.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.tipoEntradaService.findAll();
  }

  @Serealize(InstitucionTipoEntradaOutputDto)
  @Get('tipos-entrada')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna todos los tipos de entradas de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución',
    name: 'id_institucion',
    type: 'string',
  })
  tiposEntrada(@Request() req, @Query() query: IdInstitucionDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.tipoEntradaService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Serealize(InstitucionTipoEntradaMinOutputDto)
  @Get('tipos-entrada-mostrar')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna todos los tipos de entrada de una institución que tengan el campo mostrar en true.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución',
    name: 'id_institucion',
    type: 'string',
  })
  tipoEntradaMostrar(@Request() req, @Query() query: IdInstitucionDto) {
    const usuario: Usuario = req.user.usuario;

    if (!usuario)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.tipoEntradaService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
      true,
    );
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que actualiza la información de el tipo de entrada de una institución',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: {
      ejemplo: { value: { id_institucion_tipo_entrada: 130, mostrar: true } },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionTipoEntradaDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.tipoEntradaService.update(admin, body);
  }
}
