import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionService } from './institucion.service';
import { Operador } from 'src/operador/entity/operador.entity';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { UpdateInstitucionDto } from './dto/input/update.dto';
import { InstitucionOutputDto } from './dto/output/institucion.dto';
import { InstitucionMinOutputDto } from './dto/output/institucion-min.dto';
import { InstitucionesOutputDto } from './dto/output/instituciones.dto';
import { InstitucionesMinOutputDto } from './dto/output/instituciones-min.dto';

@Controller('institucion')
@ApiTags('institucion')
export class InstitucionController {
  constructor(private institucionService: InstitucionService) {}

  @Serealize(InstitucionesMinOutputDto)
  @Get('instituciones-activas')
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones activas.',
  })
  activas() {
    return this.institucionService.findAll(true);
  }

  @Serealize(InstitucionMinOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones.',
  })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    if (!superAdmin || superAdmin.tipoUsuario.id_tipo_usuario != 2)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.institucionService.findAll();
  }

  @Serealize(InstitucionOutputDto)
  @Get('institucion')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna la información de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  institucion(@Request() req, @Query() query: IdInstitucionDto) {
    const admin: Operador = req.user.operador;

    if (
      !admin ||
      (admin.tipoUsuario.id_tipo_usuario != 2 &&
        admin.tipoUsuario.id_tipo_usuario != 3)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.institucionService
      .findById(parseInt(query.id_institucion))
      .then((institucion) => {
        if (
          admin.tipoUsuario.id_tipo_usuario === 3 &&
          admin.institucion.id_institucion != institucion.id_institucion
        )
          throw new ForbiddenException(
            'No puedes acceder a esta información porque no perteneces a esta institución.',
          );
        return institucion;
      });
  }

  @Serealize(InstitucionesOutputDto)
  @Get('instituciones')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones.',
  })
  @ApiBearerAuth('jwt')
  instituciones(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    if (!superAdmin || superAdmin.tipoUsuario.id_tipo_usuario != 2)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.institucionService.findAll();
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que actualiza la información de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Todas las variables a excepción de id_institucion son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion: 200,
          _activo: true,
          _dias_multa_retraso: 7,
          _dominio: '@pcpuma.acatlan.unam.mx',
          _email_institucional: true,
          _responsable: '',
          _telefono: '',
          _tiempo_entrega: 15,
          _tiempo_prestamo: 120,
          _tiempo_recoger: 10,
          _ubicacion: '',
        },
      },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionDto) {
    const admin: Operador = req.user.operador;

    if (
      !admin ||
      (admin.tipoUsuario.id_tipo_usuario != 2 &&
        admin.tipoUsuario.id_tipo_usuario != 3)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionService.update(admin, body);
  }
}
