import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { ModuloService } from './modulo.service';
import { Operador } from 'src/operador/entity/operador.entity';
import { Usuario } from 'src/usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { CreateModuloDto } from './dto/input/create.dto';
import { ModuloDto } from './dto/input/modulo.dto';
import { UpdateModuloDto } from './dto/input/update.dto';
import { ModuloOutputDto } from './dto/output/modulo.dto';
import { ModuloMinOutputDto } from './dto/output/modulo-min.dto';

@Controller('modulo')
@ApiTags('modulo')
export class ModuloController {
  constructor(private moduloService: ModuloService) {}

  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que crea un módulo.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { id_institucion: 200, modulo: '' } } },
  })
  create(@Request() req, @Body() body: CreateModuloDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.moduloService.create(admin, body.modulo);
  }

  @Serealize(ModuloOutputDto)
  @Get('modulo')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna la información de un módulo.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id del modulo.',
    name: 'id_modulo',
    type: 'string',
  })
  modulo(@Request() req, @Query() query: ModuloDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.moduloService.findById(parseInt(query.id_modulo));
  }

  @Serealize(ModuloOutputDto)
  @Get('modulos')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todos los módulos de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  modulos(@Request() req, @Query() query: IdInstitucionDto) {
    const operador: Operador = req.user.operador;

    if (
      !operador ||
      (operador.tipoUsuario.id_tipo_usuario != 2 &&
        operador.tipoUsuario.id_tipo_usuario != 3 &&
        operador.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para acceder a esta información.',
      );
    return this.moduloService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Serealize(ModuloMinOutputDto)
  @Get('modulos-activos')
  @ApiOperation({
    description: 'Endpoint que retorna todos los módulos de una institución.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  modulosActivos(@Query() query: IdInstitucionDto) {
    return this.moduloService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
      true,
    );
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que actualiza la información de un módulo.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Todas las variables a excepción de id_modulo son opcionales.',
    examples: {
      ejemplo: { value: { id_modulo: 1, _activo: true, _modulo: '' } },
    },
  })
  update(@Request() req, @Body() body: UpdateModuloDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.moduloService.update(admin, body);
  }
}
