import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Recaptcha } from '@nestlab/google-recaptcha';
import { Serealize } from '../interceptors/serialize.interceptor';
import { UsuarioService } from './usuario.service';
import { DgaeInputDto } from './dto/input/dgae.dto';
import { DgpInputDto } from './dto/input/dgp.dto';
import { RegistrarUsuarioInputDto } from './dto/input/registrar.dto';
import { UpdateUsuarioInputDto } from './dto/input/update.dto';
import { UsuarioInputDto } from './dto/input/usuario.dto';
import { UsuariosInputDto } from './dto/input/usuarios.dto';
import { UsuarioInstitucionCarreraOutputDto } from '../institucion-usuario/dto/output/usuario-institucion-carrera.dto';
import { UsuarioOutputDto } from './dto/output/usuario.dto';
import { UsuariosOutputDto } from './dto/output/usuarios.dto';
import { Operador } from 'src/operador/entity/operador.entity';

@Controller('usuario')
@ApiTags('usuario')
export class UsuarioController {
  constructor(private usuarioService: UsuarioService) {}

  @Serealize(UsuarioInstitucionCarreraOutputDto)
  @Recaptcha()
  @Get('dgae')
  @ApiOperation({
    description: 'Endpoint que retorna información de dgae de un alumno.',
  })
  @ApiQuery({
    description: 'Número de cuenta del alumno.',
    name: 'usuario',
    type: 'string',
  })
  @ApiQuery({
    description: 'Id de la carrera institución a la que pertenece el alumno.',
    name: 'id_institucion_carrera',
    type: 'string',
  })
  dgae(@Query() query: DgaeInputDto) {
    return this.usuarioService.DGAE(
      query.usuario,
      parseInt(query.id_institucion_carrera),
    );
  }

  @Serealize(UsuarioInstitucionCarreraOutputDto)
  @Recaptcha()
  @Get('dgp')
  @ApiOperation({
    description: 'Endpoint que retorna información de dgp de un profesor.',
  })
  @ApiQuery({
    description: 'Número de cuenta del profesor.',
    name: 'usuario',
    type: 'string',
  })
  @ApiQuery({ description: 'RFC del profesor.', name: 'rfc', type: 'string' })
  dgp(@Query() query: DgpInputDto) {
    return this.usuarioService.DGP(
      parseInt(query.id_institucion),
      query.usuario,
      query.rfc,
    );
  }

  @Post('registrar')
  @Recaptcha()
  @ApiOperation({
    description: 'Enpoint que realiza el registro de un usuario.',
  })
  @ApiBody({
    description:
      'Todas la variables a excepción de id_usuario son obligatorias.',
    examples: {
      ejemplo: { value: { id_usuario: 1, telefono: '', correo: '' } },
    },
  })
  registrar(@Body() body: RegistrarUsuarioInputDto) {
    return this.usuarioService.registrar(
      body.id_institucion_usuario,
      body.telefono,
      body.correo,
    );
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Enpoint que actualiza la información de un usuario.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Todas las variables a excepción de id_usuario son opcionales.',
    examples: {
      ejemplo: {
        value: {
          correo: '',
          id_usuario: 1,
          telefono: '',
        },
      },
    },
  })
  update(@Request() req, @Body() body: UpdateUsuarioInputDto) {
    const admin: Operador = req.user.operador;

    if (
      !admin ||
      (admin.tipoUsuario.id_tipo_usuario != 3 &&
        admin.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.usuarioService.update(body);
  }

  @Put('update-password')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Enpoint que envía una nueva password al correo del usuario.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Es obligatorio enviar el campo id_usuario.',
    examples: { ejemplo: { value: { id_usuario: 1 } } },
  })
  passwordResset(@Request() req, @Body() body: UpdateUsuarioInputDto) {
    const admin: Operador = req.user.operador;

    if (
      !admin ||
      (admin.tipoUsuario.id_tipo_usuario != 3 &&
        admin.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.usuarioService.passwordReset(body.id_usuario);
  }

  @Serealize(UsuarioOutputDto)
  @Get('usuario')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Enpoint que retorna la información de un usuario.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'El número de cuenta que se quiere buscar.',
    name: 'usuario',
    type: 'string',
  })
  usuario(@Request() req, @Query() query: UsuarioInputDto) {
    const admin: Operador = req.user.operador;

    if (
      !admin ||
      (admin.tipoUsuario.id_tipo_usuario != 3 &&
        admin.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.usuarioService.findByUsuario(query.usuario, true, true);
  }

  @Serealize(UsuariosOutputDto)
  @Get('usuarios')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna 25 usuarios dependiendo de la página en la que se encuentra el usuario y sus filtros.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Id de la institución que se quiere usar como filtro.',
    name: 'id_institucion',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id de la carrera que se quiere usar como filtro.',
    name: 'id_carrera',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo usuario que se quiere usar como filtro.',
    name: 'id_tipo_usuario',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Nombre del usuario que se quiere buscar.',
    name: 'nombre',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Usuario que se quiere buscar.',
    name: 'usuario',
    type: 'string',
    required: false,
  })
  usuarios(@Request() req, @Query() query: UsuariosInputDto) {
    const operador: Operador = req.user.operador;

    if (
      !operador ||
      (operador.tipoUsuario.id_tipo_usuario != 2 &&
        operador.tipoUsuario.id_tipo_usuario != 3 &&
        operador.tipoUsuario.id_tipo_usuario != 4)
    )
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.usuarioService.findAll(query);
  }
}
